// This file keeps track of functions that create mathematic environments.

// We need the name of our environments.
#import "constants.typ": *

/* Hides a command in text. */
#let hide-cmd(cmd) = [#place(right, cmd)]

/* Emphasize a block by placing it in a decorated rectangle. */
#let block-emph(body, way: "") = {
  let outset = 5pt
  let ash(n) = rgb(185, 185, 200, n + 50)
  set rect(width: 100%, radius: 1pt, outset: outset)

  if way == "fill" {
    v(outset)
    rect(fill: ash(5))[#body]
    v(outset)
  } else if way == "outline" {
    v(outset)
    rect(stroke: ash(30) + 5pt)[#body]
    v(outset)
  } else {
    [#body]
  }
}

/* The `name` named variable allows us to give a name to be displayed instead of the numbers. */
#let math-block(body, name: "", id: "", emphasis: "bold", decoration: "fill") = {
  let current-counter = counter(id)
  let cmd = current-counter.step()

  let number = counter(heading).display() + current-counter.display()
  let figure-name = if name == "" { [#number] } else { [#number (#name)] }

  let math-figure(body) = figure(supplement: none, kind: id, numbering: it => figure-name, body)
  let name = if name == "" { [] } else { [ (#name)] }

  return if emphasis == "bold" {
    math-figure(block-emph(way: decoration)[#hide-cmd(cmd)*#id #number#name.* #body])
  } else if emphasis == "italics" {
    math-figure(block-emph(way: decoration)[#hide-cmd(cmd)_#id #number#name._ #body])
  } else {
    math-figure(block-emph(way: decoration)[#hide-cmd(cmd)#id #number#name. #body])
  }
}

#let v-space = v(0.5em)

// Functions we will directly use.
#let proposition(body, name: "") = math-block(name: name, id: proposition-id, body)
#let       lemma(body, name: "") = math-block(name: name, id: lemma-id,       body)
#let     theorem(body, name: "") = math-block(name: name, id: theorem-id,     body)
#let   corollary(body, name: "") = math-block(name: name, id: corollary-id,   body)

#let definition(body, name: "") = math-block(name: name, id: definition-id, decoration: "outline", body)

#let exercise(body, name: "") = math-block(name: name, id: exercise-id, decoration: "none", body + v-space)
#let question(body, name: "") = math-block(name: name, id: question-id, decoration: "none", body + v-space)

#let  remark(body, name: "") = math-block(name: name, id: remark-id,  emphasis: "italics", decoration: "none", body + v-space)
#let example(body, name: "") = math-block(name: name, id: example-id, emphasis: "italics", decoration: "none", body + v-space)

/* It takes the body of the proof, and formats it by adding the QED
   symbol at the end, and the _#proof-id._ text at the front. */
#let proof(body) = [_#proof-id._ #body #h(1fr) #qed#v-space]

/* Number the equations starting with the heading hierarchy before them.
   E.g. if we are looking at the 6th equation in the 4th subheading
   of the 2nd heading, it will show "(2.4.6)" in the right. */
#let numbering-equation(equation) = [(#counter(heading).display()#equation)]
#let numbered-equation(body, name: "") = {
  if name == "" {
    math.equation(numbering: numbering-equation, block: true)[#body]
  } else {
    let cmd = counter(math.equation).update(n => calc.max(0, n - 1))
    math.equation(numbering: it => "(" + name + ")", block: true)[#body#hide-cmd(cmd)]
  }
}
